#import numpy


import GEOM
import geompy
import math
import SALOMEDS
import salome
import SMESH
import smesh
import NETGENPlugin
import StdMeshers


def to_string(in_int):
    """Converts an integer to a string"""
    out_str = ""
    prefix = ""
    if in_int < 0:
        prefix = "-"
        in_int = -in_int
    while in_int / 10 != 0:
        out_str = chr(ord('0') + in_int % 10) + out_str
        in_int = in_int / 10
    out_str = chr(ord('0') + in_int % 10) + out_str
    return prefix + out_str


x0 = 0
x1 = 2
y1 = 2
z1 = 3

pcyl = geompy.MakeVertex(x0, y1, z1)
p2 = geompy.MakeVertex(x1, y1, z1)
Vcyl = geompy.MakeLineTwoPnt(pcyl, p2)
block = geompy.MakeBoxDXDYDZ(2, 3.5, 20)

geompy.addToStudy( pcyl, "pcyl" )
geompy.addToStudy( p2, "p2" )
geompy.addToStudy( Vcyl, "Vcyl" )
geompy.addToStudy( block, "block" )

Rcylinder = 0.45
Lcylinder = 2.00
clind = geompy.MakeCylinder(pcyl, Vcyl, Rcylinder, Lcylinder)
blockcyl = geompy.MakeCut(block, clind)

geompy.addToStudy( clind, "clind" )
geompy.addToStudy( block, "block" )
geompy.addToStudy( Vcyl, "Vcyl" )
geompy.addToStudy( blockcyl, "blockcyl" )

Pbot = geompy.CreateGroup(blockcyl, geompy.ShapeType["FACE"])
geompy.UnionIDs(Pbot, [13])
Ptop = geompy.CreateGroup(blockcyl, geompy.ShapeType["FACE"])
geompy.UnionIDs(Ptop, [28])
Pcyl = geompy.CreateGroup(blockcyl, geompy.ShapeType["FACE"])
geompy.UnionIDs(Pcyl, [41])

Ltop = geompy.CreateGroup(blockcyl, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Ltop, [32])
Lcyl = geompy.CreateGroup(blockcyl, geompy.ShapeType["EDGE"])
geompy.UnionIDs(Lcyl, [35])



geompy.UnionIDs(Lcyl, [3])
geompy.addToStudyInFather( blockcyl, Pbot, "Pbot" )
geompy.addToStudyInFather( blockcyl, Pcyl, "Pcyl" )
geompy.addToStudyInFather( blockcyl, Ptop, "Ptop" )
geompy.addToStudyInFather( blockcyl, Ltop, "Ltop" )
geompy.addToStudyInFather( blockcyl, Lcyl, "Lcyl" )


## mesh part
## mesh blockcyl
Mbcyl = smesh.Mesh(blockcyl)
Netgen_1D_2D_3D = Mbcyl.Tetrahedron(algo=smesh.FULL_NETGEN)
NETGEN_3D_Parameters = Netgen_1D_2D_3D.Parameters()
NETGEN_3D_Parameters.SetMaxSize( 2 )
NETGEN_3D_Parameters.SetSecondOrder( 1 )
NETGEN_3D_Parameters.SetOptimize( 1 )
NETGEN_3D_Parameters.SetFineness( 2 )
isDone = Mbcyl.Compute()

Pbot = Mbcyl.Group(Pbot)
Pcyl = Mbcyl.Group(Pcyl)
Ptop = Mbcyl.Group(Ptop)
Ltop = Mbcyl.Group(Ltop)
Lcyl = Mbcyl.Group(Lcyl)


smesh.SetName(Mbcyl.GetMesh(), 'Mbcyl')
smesh.SetName(Netgen_1D_2D_3D.GetAlgorithm(), 'Netgen 1D-2D-3D')
smesh.SetName(NETGEN_3D_Parameters, 'NETGEN 3D Parameters')


smesh.SetName(Pbot, 'Pbot')
smesh.SetName(Pcyl, 'Pcyl')
smesh.SetName(Ptop, 'Ptop')
smesh.SetName(Ltop, 'Ltop')
smesh.SetName(Lcyl, 'Lcyl')

#Ltop_2 = Mbcyl.Group(Ltop)
#Ltop_3 = Mbcyl.GroupOnGeom(Ltop,'Ltop',SMESH.NODE)


aFilterManager = smesh.CreateFilterManager()
aFilterA = aFilterManager.CreateFilter()
aCriteria = []
aCriterion = SMESH.Filter.Criterion(18,32,0,'Pcyl','',32,32,0.001,SMESH.NODE,-1)
aBelongToCylinderA = aFilterManager.CreateBelongToCylinder()
aBelongToCylinderA.SetTolerance(0.001)
aCriteria.append(aCriterion)
aFilterA.SetCriteria(aCriteria)
aFilterA.SetPredicate(aBelongToCylinderA)
Ntop = Mbcyl.CreateEmptyGroup( SMESH.NODE, 'Ntop' )

nbAdd = Ntop.Add( [ 1, 4, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 453, 454, 455, 456, 457, 458, 459, 460, 461, 462, 463, 464, 465, 466, 467, 468, 469, 470, 471, 472, 473, 474, 475, 476, 477, 478, 479, 480, 481, 482, 549, 550, 551, 552, 553, 554, 555, 556, 557, 558, 559, 560, 561, 562, 563, 564, 565, 566, 567, 568, 569, 570, 571, 572, 573, 574, 575, 576, 577, 578, 579, 580, 581, 582, 583, 584, 585, 586, 587, 588, 589, 590, 591, 592, 593, 594, 595, 596, 597, 598, 599, 600, 601, 602, 603, 604, 605, 606, 607, 608, 609, 610, 611, 612, 613, 614, 615, 616, 617, 618, 619, 620, 621, 622, 623, 624, 625, 626, 627, 628, 629, 630, 631, 632, 633, 634, 635, 636, 637, 638, 639, 640, 641, 642, 643, 644, 645, 646, 647, 648, 649, 650, 651, 652, 653, 654, 655, 656, 657, 658, 659, 660, 661, 662, 663, 664, 665, 666, 667, 668, 669, 670, 671, 672, 673, 674, 675, 676, 677, 678, 679, 680, 681, 682, 683, 684, 685, 686, 687, 688, 689, 690, 691, 692, 693, 694, 695, 696, 697, 698, 699, 700, 701, 702, 703, 704, 705, 706, 707, 708, 709, 710, 711, 712, 713, 714, 715, 716, 717, 718, 719, 720, 721, 722, 723, 724, 725, 726, 727, 728, 729, 730, 731, 732, 733, 734, 735, 736, 737, 738, 739, 740, 741, 742, 743, 744, 745, 746, 747, 748, 749, 750, 751, 752, 753, 754, 755, 756, 757, 758, 759, 760, 761, 762, 763, 764, 765, 766, 767, 768, 769, 770, 771, 772, 773, 774, 775, 776, 777, 778, 779, 780, 781, 782, 783, 784, 785, 786, 787, 788, 789, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799, 800, 801, 802, 803, 804, 805, 806, 807, 808 ] )



countall=0
countcyl=0
node_oncyl = []
Rcyl2 = Rcylinder**2
eps = 1e-4
zoffset = 0.0  ## change to 10.0 for second picture
print "\ncopy and paste next part into command file",
print "\n******************************************\n"
print "cylco2=AFFE_CHAR_MECA(MODELE=Cmod,\n     LIAISON_DDL = (",
for allnode in Mbcyl.GetNodesId():
    countall+=1
    q1, q2, q3 = Mbcyl.GetNodeXYZ(allnode)
    # on cylinder yes/no:
    dy2 = (q2-y1)**2
    dz2 = (q3-z1)**2
    if abs(dy2+dz2-Rcyl2)<eps:
       countcyl+=1
       cnode = to_string(allnode)
       node_oncyl.append(allnode)
       #ss = []
       ss = "\n          _F(NOEUD=('N"+cnode+"','N"+cnode+"'),DDL=('DY','DZ'),COEF_MULT=("
       print ss,
       print (q2-y1),
       print ",",
       print (q3-z1-zoffset),
       print "),COEF_IMPO=0.00),",
    pass
pass
print "),);"
print "\nend of copy and past",
print "\n********************\n\n\n"

print "all nodes - cylinder nodes: ",countall,countcyl



if salome.sg.hasDesktop():
		salome.sg.updateObjBrowser(0)


